/*
    <Interfaz Grafica>
    Copyright (C) <2011>  <José Antonio Sánchez Reynaga>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "window.h"
#include "ui_window.h"


Window::Window(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Window)
{
    ui->setupUi(this);
    
    //EVENTOS
    connect(ui->cb_devices, SIGNAL(activated(int)), this, SLOT(refreshDeviceProperties()));
    
    //Respecto al hilo
    threadFormat = new ThreadFormat(this);
    connect(threadFormat, SIGNAL(started()), this, SLOT(disableGUI()));
    connect(threadFormat, SIGNAL(finished()), this, SLOT(enableGUI()));
    connect(threadFormat, SIGNAL(finished()), this, SLOT(checkFormat()));
    
    //Comenzar a formatear
    connect(ui->but_apply, SIGNAL(clicked(bool)), this, SLOT(applyFormat()));
    
    connect(ui->but_refresh_devices, SIGNAL(clicked(bool)), this, SLOT(refreshListDevices()));
    
     //Cargamos Los Dispositivos por primera vez
    refreshListDevices();
    
    //Cargamos los sistemas de archivos disponibles
    ui->cb_filesystem->addItem("fat32");
    ui->cb_filesystem->addItem("ntfs");
    ui->cb_filesystem->addItem("ext4");
    ui->cb_filesystem->addItem("ext3");
    ui->cb_filesystem->addItem("ext2");
    
}

Window::~Window()
{

}



//SLOTS

void Window::refreshListDevices()
{
    
    
    //Solo Hacer Cambios si el hilo no esta corriendo
    if(threadFormat->isRunning())
        return;
    
    ui->cb_devices->clear();
    
    QMap <QString, Device> USBs = Devices::listAvaliableDevices();
    
    //No hagas algo cuando no hay dispositivos
    if(USBs.count() == 0){
         // Reiniciamos etiquetas de propiedades
        ui->lb_product->setText( i18n("Product: "));
        ui->lb_size->setText( i18n("Capacity: " ) );
        ui->lb_filesystem->setText( i18n("Current filesystem: ") );
        
        ui->lb_notice->setText( i18n("No devices found") );
        
        disableGUI();
        
        return;
    }
   
    
   
   //Agregamos valores al combobox
    QMapIterator<QString, Device> iterador(USBs);
    while(iterador.hasNext()){
       iterador.next();
       
       ui->cb_devices->addItem(iterador.key());
       
    }
    
    //Al final mostramos las propiedades del dispositivo
    refreshDeviceProperties();
    
}

void Window::refreshDeviceProperties()
{
   
    if(ui->cb_devices->count() == 0){
         // Reiniciamos etiquetas de propiedades
        ui->lb_product->setText( i18n("Product: "));
        ui->lb_size->setText( i18n("Capacity: " ) );
        ui->lb_filesystem->setText( i18n("Current filesystem: ") );
        
        ui->lb_notice->setText( i18n("Device not found") );
        
        disableGUI();
        
        return;
    }
    
    //Obtenemos informacion sobre el combobox actual
    
    QString dir_device= ui->cb_devices->currentText();

    //Obtenemos el objeto Device Actual
    actual = Devices::listAvaliableDevices()[dir_device];
    
    //rellenamos datos
    
    QString giga = QVariant(actual.as<StorageVolume>()->size()).toString();
    float g = giga.toFloat();
    int gig = (int) (g / 1024) / 1024;
    
    ui->lb_product->setText( i18n("Product: ") + actual.product() );
    ui->lb_size->setText( i18n("Capacity: " )  + QVariant(gig).toString()+ " MB");
    ui->lb_filesystem->setText( i18n("Current filesystem: ") + actual.as<StorageVolume>()->fsType() );
    
    
    
    enableGUI();
    ui->lb_notice->setText("");
}


void Window::applyFormat()
{
    //Confirmar, el formateo del dispositivo
    int resp = QMessageBox::question(this, i18n("Warning"), i18n("Do you really want to format the device?"), QMessageBox::Yes, QMessageBox::Cancel);
    
    if(resp == QMessageBox::Cancel){
        return;
    }
    
    
    //Verificamos si se necesita crear particion
    if(!actual.as<Block>()->device().contains(QRegExp("([0-9])$"))){
        qDebug() << "WARNING: It is needed to create a partition";
        ui->lb_notice->setText(i18n("It is needed to create a partition"));
        return;
        
    }
    
    
    // El dispositivo esta montando
    if(actual.as<StorageAccess>()->isAccessible()){
          qDebug() << ">> Volume is mounted, trying to unmount it";
          // tratar de desmontarlo
           if(actual.as<StorageAccess>()->teardown()){
               qDebug() << "  >> Volume as been unmounted";
           }
           else{
            //Aqui no se pudo desmontarlo
            ui->lb_notice->setText(i18n("Before we continue, device will be unmounted"));
            return;
           }
    }
    else {
            qDebug() << ">> Volume is unmounted";
    }
    
    
    //Establecemos los argumentos desde la GUI
    
    threadFormat->setFilesystem( ui->cb_filesystem->currentText() ); //Sistema de archivos
    threadFormat->setDirDev(actual.as<Block>()->device()); //Ubicacion de la memoria
    threadFormat->setLabel(ui->le_label_device->text());
    
    //Todo bien, comenzar formateo
    ui->lb_notice->setText(i18n("Formatting USB device ..."));
    threadFormat->start();
}

void Window::disableGUI()
{
    ui->widget_2->setDisabled(true);
    ui->but_apply->setDisabled(true);
    ui->widget_4->setDisabled(true);
    ui->widget_5->setDisabled(true);
}

void Window::enableGUI()
{
    ui->widget_2->setEnabled(true);
    ui->but_apply->setDisabled(false);;
    ui->widget_4->setEnabled(true);
    ui->widget_5->setEnabled(true);
}

//Cuando el hilo termina la ejecucion se ejecuta este metodo
void Window::checkFormat()
{
    //Verificamos como quedo la ejecucion del formato
    if(threadFormat->isOk()){
        ui->lb_notice->setText("");
        
        //Mostramos un mensaje para darle instrucciones al usuario
        QMessageBox::information(
            this,  //Widget padre
            i18n("USB device formated"), //titulo
                                 //Contenido del dialogo
	    i18n("Your USB device was successfully formated. Please, unplug it and plug it again to use it."),
                                 QMessageBox::Apply
                                );
        
            //Cerramos el programa, ya no hay nada que hacer
            this->close();
        
    }
    else{
        ui->lb_notice->setText(i18n("Formating process failed ..."));
    }

    
}

void Window::setDevice(Device dev)
{
    actual = dev;
}


Ui::Window* Window::getUi()
{
    return ui;
}



